define(['exports', 'module', './focus-relevant', './valid-area', './visible', './disabled', './only-tabbable', '../util/tabindex-value'], function (exports, module, _focusRelevant, _validArea, _visible, _disabled, _onlyTabbable, _utilTabindexValue) {
  // determine if an element can be focused

  // http://www.w3.org/TR/html5/editing.html#focus-management

  // NOTE: The following known issues exist:
  //   Gecko: `svg a[xlink|href]` is not identified as focusable (because SVGElement.prototype.focus is missing)
  //   Blink, WebKit: SVGElements that have been made focusable by adding a focus event listener are not identified as focusable

  'use strict';

  function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

  var _isFocusRelevant = _interopRequireDefault(_focusRelevant);

  var _isValidArea = _interopRequireDefault(_validArea);

  var _isVisible = _interopRequireDefault(_visible);

  var _isDisabled = _interopRequireDefault(_disabled);

  var _isOnlyTabbable = _interopRequireDefault(_onlyTabbable);

  var _tabindexValue = _interopRequireDefault(_utilTabindexValue);

  function isOnlyFocusRelevant(element) {
    var nodeName = element.nodeName.toLowerCase();
    if (nodeName === 'embed' || nodeName === 'keygen') {
      // embed is considered focus-relevant but not focusable
      // see https://github.com/medialize/ally.js/issues/82
      return true;
    }

    var _tabindex = (0, _tabindexValue['default'])(element);
    if (element.shadowRoot && _tabindex === null) {
      // Shadow DOM host elements *may* receive focus
      // even though they are not considered focuable
      return true;
    }

    if (nodeName === 'area') {
      // all <area>s are considered relevant,
      // but only the valid <area>s are focusable
      return !(0, _isValidArea['default'])(element);
    }

    return false;
  }

  module.exports = function (element) {
    if (element === document) {
      element = document.documentElement;
    }

    if (!element || element.nodeType !== Node.ELEMENT_NODE) {
      throw new TypeError('is/focusable requires an argument of type Element');
    }

    if (!(0, _isFocusRelevant['default'])(element) || isOnlyFocusRelevant(element)) {
      return false;
    }

    if ((0, _isDisabled['default'])(element)) {
      return false;
    }

    if ((0, _isOnlyTabbable['default'])(element)) {
      // some elements may be keyboard focusable, but not script focusable
      return false;
    }

    // elements that are not rendered, cannot be focused
    if (!(0, _isVisible['default'])(element)) {
      return false;
    }

    return true;
  };
});
//# sourceMappingURL=focusable.js.map