define(['exports', 'module', './native-disabled-supported.supports'], function (exports, module, _nativeDisabledSupportedSupports) {
  // Determine if an element supports the disabled attribute

  'use strict';

  function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

  var _supports2 = _interopRequireDefault(_nativeDisabledSupportedSupports);

  var supports = undefined;

  // http://www.w3.org/TR/html5/disabled-elements.html#concept-element-disabled
  var disabledElementsPattern = undefined;
  var disabledElements = {
    input: true,
    select: true,
    textarea: true,
    button: true,
    fieldset: true,
    form: true
  };

  module.exports = function (element) {
    if (!supports) {
      supports = (0, _supports2['default'])();

      if (supports.canFocusDisabledFieldset) {
        delete disabledElements.fieldset;
      }

      if (supports.canFocusDisabledForm) {
        delete disabledElements.form;
      }

      disabledElementsPattern = new RegExp('^(' + Object.keys(disabledElements).join('|') + ')$');
    }

    if (!element || element.nodeType !== Node.ELEMENT_NODE) {
      throw new TypeError('is/native-disabled-supported requires an argument of type Element');
    }

    var nodeName = element.nodeName.toLowerCase();
    return Boolean(disabledElementsPattern.test(nodeName));
  };
});
//# sourceMappingURL=native-disabled-supported.js.map